/*
 * MPDclient - Music Player Daemon (MPD) remote for Mac OS
 * Copyright (C) 2019  Old-Geek
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */


#import "MPDCommand.h"
#import "MPDSocket.h"
#import "MPDCache.h"
#import "MPDAlbum.h"
#import "MPDUtils.h"
#import "MPDServerError.h"
#import "Song.h"
#import "Album.h"


@implementation MPDCommand

static MPDCommand *sMPDCommandManager = nil;
struct mpd_connection *mpdConnection = nil;
struct mpd_status * mpdConnectionStatus;

MPDSocket *mpdSocket;


// Singleton
+ (MPDCommand *)sMPDCommand {
    if (sMPDCommandManager == nil)
        sMPDCommandManager = [[super allocWithZone:NULL] init];
    
    return sMPDCommandManager;
}

+ (id)allocWithZone:(NSZone *)zone {
    return [self sMPDCommand];
}

- (id)copyWithZone:(NSZone *)zone {
    return self;
}


- (id) init {
    self = [super init];
    mpdSocket = [MPDSocket sMPDSocket];
    mpdConnection = [mpdSocket getConnection];
    return self;
}

- (void)connect {
    [mpdSocket connect];
    mpdConnection = [mpdSocket getConnection];
}

- (void)disconnect {
    [mpdSocket disconnect];
}


- (void)play {
    
    if (![mpdSocket isConnected])
        return;
    
    MPDStatus *status = [[MPDCache sMPDCache] getStatus ];
    
    if (status.state == MPD_STATE_STOP) {
        mpd_run_play(mpdConnection);
        // force la mise à jour du titre si MPD est sur STOP
        [[MPDCache sMPDCache]resetStatus];
    } else if (status.state == MPD_STATE_PLAY) {
        mpd_run_toggle_pause(mpdConnection);
    } else if (status.state == MPD_STATE_PAUSE) {
        mpd_run_toggle_pause(mpdConnection);
    } else {
        mpd_run_play(mpdConnection);
    }
}

- (void)stop {
    if ([mpdSocket isConnected])
        mpd_run_stop(mpdConnection);
}

- (void)next {
    if (![mpdSocket isConnected])
        return;
    
    mpd_send_status(mpdConnection);
    mpdConnectionStatus = mpd_recv_status(mpdConnection);
        
    if (mpdConnectionStatus != NULL) {
            
        int queueLength = mpd_status_get_queue_length(mpdConnectionStatus);
        int state = mpd_status_get_state(mpdConnectionStatus);
            
        if (state == MPD_STATE_PLAY || state == MPD_STATE_PAUSE) {
                
            int songPosition = mpd_status_get_song_pos(mpdConnectionStatus);
                
            mpd_status_free(mpdConnectionStatus);
                
            if (songPosition < queueLength-1)
                mpd_run_next(mpdConnection);
        }
    }
}

- (void)previous {
    if (![mpdSocket isConnected])
        return;
        
    MPDStatus *status = [[MPDCache sMPDCache] getStatus ];
        
    if ((status.state == MPD_STATE_PLAY || status.state == MPD_STATE_PAUSE) && status != nil)
        mpd_run_previous(mpdConnection);
    
}


- (void)setVolume:(unsigned int)aVol {
    if ([mpdSocket isConnected])
        mpd_run_set_volume(mpdConnection, aVol);
}

- (void)seekPosition:(float)aPosition {
    if ([mpdSocket isConnected] && [[MPDCache sMPDCache]getStatus].state == MPD_STATE_PLAY )
        mpd_run_seek_current(mpdConnection, aPosition, false);
}


- (void)playAlbum:(NSString *)aAlbum path:(NSString *)aPath position:(unsigned int)aPosition {
    if (![mpdSocket isConnected])
        return;
        
    NSMutableArray *albumList = [NSMutableArray array];
    const char *request = [aPath UTF8String];
    struct mpd_entity * entity;
    mpd_send_list_meta(mpdConnection, request);
        
    while ((entity = mpd_recv_entity(mpdConnection)) != NULL) {
        const struct mpd_song *song;
            
        if (mpd_entity_get_type(entity) == MPD_ENTITY_TYPE_SONG) {
            song = mpd_entity_get_song(entity);
                
            if ([aAlbum isEqualToString:[GetTag song:song tag:MPD_TAG_ALBUM]]) {
                    
                NSString *path = [NSString stringWithFormat:@"%@", [NSString stringWithCString:mpd_song_get_uri(song) encoding:NSUTF8StringEncoding]];
                    
                [albumList addObject:path];
            }
        }
    }
    
    [self playAlbum:albumList:aPosition];
}


- (void)playAlbum:(NSArray *)aList :(unsigned int)aPosition {
    if (![mpdSocket isConnected])
        return;
        
    [self stop];
    mpd_run_clear(mpdConnection);
        
    for (NSString *obj in aList)
        mpd_run_add(mpdConnection, [obj UTF8String]);
        
    if (aPosition != -1)
        mpd_run_play_pos(mpdConnection, aPosition);
    else
        mpd_run_play(mpdConnection);
}


- (void)playQueuePosition:(unsigned int)aQp {
    if ([mpdSocket isConnected])
        mpd_run_play_pos(mpdConnection, aQp);
}


- (void)playStream:(NSString *)aUrl {
    if (![mpdSocket isConnected])
        return;
                
    const char *url = [aUrl UTF8String];
        
    mpd_run_clear(mpdConnection);
    mpd_run_add(mpdConnection, url);
    mpd_run_play(mpdConnection);
}


- (void)playPlaylist:(NSArray *)aList position:(unsigned int)aPosition {
    if (![mpdSocket isConnected])
        return;
    
    mpd_run_clear(mpdConnection);
        
    for (Song *obj in aList)
        mpd_run_add(mpdConnection, [obj.fullPath UTF8String]);
        
    mpd_run_play_pos(mpdConnection, aPosition);
}


- (void)updateDB {
    
    if (![mpdSocket isConnected])
        return;
        
    [[MPDSocket sMPDSocket] stopTimer];
    [[MPDCache sMPDCache] setCurrentAlbumList:nil];
    [[MPDCache sMPDCache] albumUpdateTrigger:@"Mise à jour de la base MPD..."];
        
    if (mpd_run_update(mpdConnection, "/") != 0)
        [self asyncUpdate];
}

- (void)asyncUpdate {
    
    MPDAlbum *mpdAlbum = [[MPDAlbum alloc] init];
    
    dispatch_async(dispatch_get_global_queue(0, 0), ^{
        
        int oldValue = -1;
        int newValue = 0;
        
        while (oldValue != newValue) {
            oldValue = newValue;
            newValue = (int)mpdAlbum.getAlbumsList.count;
            sleep(1);
        }
        
        dispatch_sync(dispatch_get_main_queue(), ^{
            
            [[MPDCache sMPDCache] setCurrentAlbumList:mpdAlbum.getAlbumsList];

            NSLog(@"disconnect");
            [self disconnect];
            NSLog(@"connect");
            [self connect];

            [[MPDCache sMPDCache] dbUpdated];
        });
    });
}


@end
