/*
 * MPDclient - Music Player Daemon (MPD) remote for Mac OS
 * Copyright (C) 2019  Old-Geek
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#import "MPDSocket.h"
#import "MPDCache.h"
#import "MPDServerError.h"
#import "MPDUtils.h"
#import "Setting.h"


@implementation MPDSocket

Setting * setting;

static MPDSocket *sMPDSocketManager = nil;
struct mpd_status * connectionStatus;
struct mpd_connection *connection;

NSTimer *timer;

BOOL isConnected = NO;

// Singleton
+ (MPDSocket *)sMPDSocket {
    
    if (sMPDSocketManager == nil)
        sMPDSocketManager = [[super allocWithZone:NULL] init];
    
    return sMPDSocketManager;
}

+ (id)allocWithZone:(NSZone *)zone {
    return [self sMPDSocket];
}

- (id)copyWithZone:(NSZone *)zone {
    return self;
}


- (id)init {
    self = [super init];
    
    setting = [[Setting alloc] init];
    [self connect];
    
    return self;
}


- (int)connect {
    const char *host = [setting.host UTF8String];
    unsigned int port = setting.port;
    
    //  const char *user = [setting.user UTF8String];
    //  const char *pass = [setting.pass UTF8String];
    
    connection = mpd_connection_new(host, port, 3000);
    
    if (mpd_connection_get_error(connection) != MPD_ERROR_SUCCESS) {
        [[MPDServerError sMPDServerError] send:connection ];
        
    } else {
        
        isConnected = YES;
        
        NSMethodSignature *sgn = [self methodSignatureForSelector:@selector(onTicTac:)];
        NSInvocation *inv = [NSInvocation invocationWithMethodSignature: sgn];
        [inv setTarget: self];
        [inv setSelector:@selector(onTicTac:)];
        
        timer = [NSTimer timerWithTimeInterval: 0.1  invocation:inv repeats:YES];
        
        NSRunLoop *runner = [NSRunLoop currentRunLoop];
        [runner addTimer: timer forMode: NSRunLoopCommonModes]; // NSDefaultRunLoopMode --> sync, NSRunLoopCommonModes async ?
    }
    
    return 0;
}

- (struct mpd_connection*)getConnection{
    return connection;
}

- (BOOL)isConnected {
    return !((mpd_connection_get_error(connection) != MPD_ERROR_SUCCESS ||!mpd_response_finish(connection)) && isConnected);
}

- (void)disconnect {
    
    if (isConnected) {
        [self stopTimer];
        mpd_connection_free(connection);
        
        isConnected = NO;
    }
}

- (void)stopTimer {
    [timer invalidate];
    timer = nil;
}

-(void)onTicTac:(NSTimer *)timer {
     [[MPDCache sMPDCache] setStatus:[self getStatus]];
}


- (MPDStatus *)getStatus {
    
    MPDStatus *mpdStatus = [[MPDStatus alloc]init];
    
    mpd_send_status(connection);
    connectionStatus = mpd_recv_status(connection);
    
    if (connectionStatus == NULL) {
        isConnected = NO;
        [self stopTimer];
        [[MPDServerError sMPDServerError] send:connection ];
        return nil;
    }
    
    unsigned int elapsedTime;
    int totalTime;
    int position;
    int state;
    unsigned volume;
    
    NSString *artiste;
    NSString *albumArtiste;
    NSString *album;
    NSString *songTitle;
    NSString *songPath;
    
    
    elapsedTime = mpd_status_get_elapsed_ms(connectionStatus);
    totalTime = mpd_status_get_total_time(connectionStatus);
    position = mpd_status_get_song_pos(connectionStatus);
    state = mpd_status_get_state(connectionStatus);
    volume =  mpd_status_get_volume(connectionStatus);
    
    
    mpd_status_free(connectionStatus);
    
    struct mpd_song *song;
    
    mpd_send_current_song(connection);
    
    while ((song = mpd_recv_song(connection)) != NULL) {
        
        artiste = [GetTag song:song tag:MPD_TAG_ARTIST];
        albumArtiste = [GetTag song:song tag:MPD_TAG_ALBUM_ARTIST];
        album = [GetTag song:song tag:MPD_TAG_ALBUM];
        songTitle = [GetTag song:song tag:MPD_TAG_TITLE];
        songPath = [NSString stringWithFormat:@"%@", [NSString stringWithCString:mpd_song_get_uri(song) encoding:NSUTF8StringEncoding]];
    }
    
    [mpdStatus setAlbum:album];
    [mpdStatus setArtist:artiste];
    [mpdStatus setAlbumArtist:albumArtiste];
    [mpdStatus setTitle:songTitle];
    [mpdStatus setPath:songPath];
    
    [mpdStatus setElapsedTime:elapsedTime];
    [mpdStatus setTotalTime:totalTime];
    [mpdStatus setTitlePosition:position];
    [mpdStatus setState:state];
    [mpdStatus setVolume:volume];
    
    return mpdStatus;
}


@end
